﻿using Battleship.DrawningObjects;
using Battleship.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Battleship.Generics
{
    internal class ShipsGenericStorage
    {
        readonly Dictionary<string, ShipGenericCollection<DrawningShip,
        DrawningObjectShip>> _shipStorages;

        public List<string> Keys => _shipStorages.Keys.ToList();

        private readonly int _pictureWidth;

        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';

        public ShipsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _shipStorages = new Dictionary<string,
            ShipGenericCollection<DrawningShip, DrawningObjectShip>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        public void AddSet(string name)
        {
            if (_shipStorages.ContainsKey(name))
                return;
            _shipStorages[name] = new ShipGenericCollection<DrawningShip, DrawningObjectShip>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_shipStorages.ContainsKey(name))
                return;
            _shipStorages.Remove(name);
        }

        public ShipGenericCollection<DrawningShip, DrawningObjectShip>?
        this[string ind]
        {
            get
            {
                if (_shipStorages.ContainsKey(ind))
                    return _shipStorages[ind];
                return null;
            }
        }
        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string,
            ShipGenericCollection<DrawningShip, DrawningObjectShip>> record in _shipStorages)
            {
                StringBuilder records = new();
                foreach (DrawningShip? elem in record.Value.GetShips)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using FileStream fs = new(filename, FileMode.Create);
            byte[] info = new
            UTF8Encoding(true).GetBytes($"ShipStorage{Environment.NewLine}{data}");
            fs.Write(info, 0, info.Length);
            return true;
        }
        /// <summary>
        /// Загрузка информации по кораблям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }
            string bufferTextFromFile = "";
            using (FileStream fs = new(filename, FileMode.Open))
            {
                byte[] b = new byte[fs.Length];
                UTF8Encoding temp = new(true);
                while (fs.Read(b, 0, b.Length) > 0)
                {
                    bufferTextFromFile += temp.GetString(b);
                }
            }
            var strs = bufferTextFromFile.Split(new char[] { '\n', '\r' }, StringSplitOptions.RemoveEmptyEntries);
            if (strs == null || strs.Length == 0)
            {
                return false;
            }
            if (!strs[0].StartsWith("ShipsStorage"))
            {
                //если нет такой записи, то это не те данные
                return false;
            }
            _shipStorages.Clear();
            foreach (string data in strs)
            {
                string[] record = data.Split(_separatorForKeyValue,
                StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 2)
                {
                    continue;
                }
                ShipGenericCollection<DrawningShip, DrawningObjectShip> collection = new(_pictureWidth, _pictureHeight);
                string[] set = record[1].Split(_separatorRecords,
                StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    DrawningShip? ship =
                    elem?.CreateDrawningShip(_separatorForObject, _pictureWidth, _pictureHeight);
                    if (ship != null)
                    {
                        if (!(collection + ship))
                        {
                            return false;
                        }
                    }
                }
                _shipStorages.Add(record[0], collection);
            }
            return true;
        }
    }
}

